100 rem + ************************************************************************ +
102 rem *<                                 FI2imp                                 >*
104 rem + ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ +
106 rem *                 Import an FI2, PSION formated, database                  *
108 rem *                                                                          *
110 rem * This script is designed to re-import an FI2 database, eg one that was    *
112 rem * previously exported with FI2MkExp from the FI2 GetDBass_bas script.      *
114 rem * It converts the export file into DATA statements that can be MERGEd      *
116 rem * into a S*BASIC program, like MakeFI2_bas, to produce a new FI2           *
118 rem * FileInfo_bin databse & extension file.                                   *
120 rem * See the mentioned scripts for further details (including why you  might  *
122 rem * want to do anything as (apparently) bizarre as this!)                    *
124 rem *                                                                          *
126 rem * Note: Only the comma-delimeted export file format (PSION) is supported   *
128 rem * by this script!                                                          *
130 rem *                                                                          *
132 rem * Dependencies: The IMPARSE command, and if wanted: QUICKSORT. Both        *
134 rem * available off my Knoware.no website.                                     *
136 rem + ------------------------------------------------------------------------ +
138 rem * V0.01, pjw, 2022 Jan 09                                                  *
140 rem + ************************************************************************ +
142 :
144 :
146 rem     Edit the two lines below according to local circumstances
148 fnm_exp$ = 'ram1_FI2db_exp':    rem Export file file name
150 fnm_dat$ = 'ram1_FI2_bas':    rem Output file file name
152 :
154 Init:                           rem Initalise GLOBal variables and arrays
156 :
158 rem     Import fi2 database
160 ert FI2import(fnm_exp$):        rem Import into arrays
162 ert FI2MkData(fnm_dat$):        rem Output DATA statements to MERGE
164 :
166 rem     Done
168 beep 2000, 2
170 quit 0
172 :
174 :
176 def proc Init
178 rem     Check for correct version
180 IF FI2_INFO$(0) <> '3.50' THEN
182  PRINT 'This is for FI2 V3.50 only': PAUSE: quit -15
184 END IF
186 :
188 rem     Init GLOBal variables and arrays
190 ext$ = 'FIv3':                          rem See FI2 manual!
192 :
194 dim wid%(7)
196 cnt% = IMPgetmetrics(fnm_exp$):         rem GLObal count!
198 if cnt% < 0: quit cnt%
200 :
202 DIM xt$(cnt%, 6)
204 DIM Exts$(cnt%, 4), Typs%(cnt%), Flgs$(cnt%, 6), Acts$(cnt%, wid%(3)), Jobs$(cnt%, wid%(4))
206 DIM Thgs$(cnt%, wid%(5)), Fnms$(cnt%, wid%(6)), Cmds$(cnt%, wid%(7))
208 enddef Init
210 :
212 :
214 def fn FI2MkData(fnm$)
216 loc i%, co
218 co = fop_over(fnm$): if co < 0: ret co
220 bput#co; 'rem FI2 data for ' & id$, 10, 58, 10
222 bput#co; "rem  Ext, Type, Flags, Action, Cjob, Thing, Xfile, Command", 10
224 :
226 for i% = 0 to cnt%
228  print#co; 'data'! '"'; Exts$(i%); '", ';
230  print#co; Typs%(i%);               ', ';
232  print#co; Flgs$(i%);               ', ';
234  print#co; '"'; Acts$(i%);         '", ';
236  print#co; '"'; Jobs$(i%);         '", ';
238  print#co; '"'; Thgs$(i%);         '", ';
240  print#co; '"'; Fnms$(i%);         '", ';
242  print#co;      Cmds$(i%)
244 endfor i%
246 close#co
248 ret 0
250 enddef FI2MkData
252 :
254 :
256 def fn FI2import(fnm$)
258 loc lp, ch, c%, s$, l$
260 ch = fop_in(fnm$): if ch < 0: ret ch
262 c% = 0
264 input#ch; l$
266 rep lp
268  if eof(#ch): er = 0: exit lp
270  input#ch; l$
272  er = imparse(s$, l$): if er < 0: exit lp
274  er = imparse(Exts$(c%), l$)
276  er = imparse(s$, l$): Typs%(c%) = s$
278  er = imparse(Flgs$(c%), l$)
280  er = imparse(Acts$(c%), l$)
282  er = imparse(Jobs$(c%), l$)
284  er = imparse(Thgs$(c%), l$)
286  er = imparse(Fnms$(c%), l$)
288  if len(l$) > 2: l$ = l$(2 to len(l$) - 1)
290  Cmds$(c%) = l$
292  c% = c% + 1
294 END REPeat lp
296 close#ch
298 ret er
300 enddef FI2import
302 :
304 :
306 def fn IMPgetmetrics(fnm$)
308 loc lp, j%, ch, c%, s$, l$
310 ch = fop_in(fnm$): if ch < 0: ret ch
312 c% = 0
314 input#ch; l$:                                   rem Skip export header
316 REPeat lp
318  if eof(#ch): er = 0: exit lp
320  input#ch; l$
322  er = imparse(s$, l$): if er < 0: exit lp:      rem Strip index, xt$
324  for j% = 0 to dimn(wid%) - 1
326   er = imparse(s$, l$): if er < 0: exit lp
328   if len(s$) > wid%(j%): wid%(j%) = len(s$)
330  endfor j%
332   j% = j% + 1: if len(l$) > wid%(j%): wid%(j%) = len(l$)
334  c% = c% + 1
336 END REPeat lp
338 close#ch
340 if er < 0: ret er
342 ret c% - 1
344 enddef IMPgetmetrics
346 :
348 :
