100 rem + ************************************************************************ +
102 rem *<                                FI2getDB                                >*
104 rem + ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ +
106 rem *                     Get database data from FileInfo2                     *
108 rem *                                                                          *
110 rem * This is to make it easier to make bulk adjustments to the FileInfo2      *
112 rem * database without having to fiddle each entry individually via            *
114 rem * FI2Config                                                                *
116 rem *                                                                          *
118 rem * Extensions entries can also be sorted alphabetically, if wanted.         *
120 rem * (Needs my QUICKSORT routine, available at Knoware.no/toolkits).          *
122 rem *                                                                          *
124 rem * Choose to export the database as a PSION-compatible Export file, or as   *
126 rem * DATA statements. The DATA staments version can be merged directly        *
128 rem * into the complimentary script MakeFI2_bas to create an entirely fresh    *
130 rem * FI2 database.                                                            *
132 rem *                                                                          *
134 rem * If you only want to append a few new entries to an existing FI2          *
136 rem * database, just use the ..FI2_util_WL_fi2inst_bas script that comes       *
138 rem * with FI2 instead.                                                        *
140 rem *                                                                          *
142 rem * Note: Keep a copy of your original FI2 database - just in case..         *
144 rem + ------------------------------------------------------------------------ +
146 rem * V0.01, pjw, 2021 Dec 17                                                  *
148 rem + ************************************************************************ +
150 :
152 :
154 filename$ = 'ram1_FI2db':               rem File name stem
156 :
158 Init:                                   rem Initialise GLOBal variables
160 :
162 rem     Read fi2 database
164 FI2read:                                rem Mandatory
166 FI2sort:                                rem Optional sort by extension (REM out)
168 :
170 rem     Choose Export or DATA (REM out unwanted ones)
172 ert FI2MkData(filename$ & '_bas'):              rem Make DATA statements to MERGE
174 rem ert FI2MkExp(filename$ & "_exp", ','):      rem Export (PSION)
176 rem ert FI2MkExp(filename$ & ".exp", chr$(9)):  rem Export (Tab delimeter)
178 :
180 rem     Done
182 beep 2000, 2
184 quit 0
186 :
188 :
190 def proc Init
192 rem     Check for correct version
194 if fi2_info$(0) <> '3.50' then
196  print 'This is for FI2 V3.50 only': pause: quit -15
198 endif
200 :
202 rem     Init GLOBal variables and arrays
204 ext$ = 'FIv3':                          rem See FI2 manual!
206 :
208 dim wid%(5)
210 cnt% = FI2getmetrics:                   rem GLObal count!
212 :
214 DIM Exts$(cnt%, 4), Thgs$(cnt%, wid%(0)), Fnms$(cnt%, wid%(1)), Cmds$(cnt%, wid%(3))
216 DIM Jobs$(cnt%, wid%(4)), Acts$(cnt%, wid%(5)), Flgs$(cnt%, 8)
218 enddef Init
220 :
222 :
224 def proc FI2sort
226 loc i%, n%, x$(6), xt$(cnt%, 6)
228 :
230 x$ = ''
232 for i% = 0 to cnt%
234  if Exts$(i%) <> x$ then
236   if Exts$(i%) = "" then
238    n% = n% + 1
240    xt$(i%) = x$ & chr$(n%)
242   else
244    x$ = Exts$(i%): n% = 65
246    xt$(i%) = x$ & chr$(n%)
248   endif
250  endif
252 endfor i%
254 quicksort xt$; 1, Exts$; 0, Thgs$; 0, Fnms$; 0, Cmds$; 0, Jobs$; 0, Acts$; 0, Flgs$; 0
256 enddef FI2sort
258 :
260 :
262 rem + ------------------------------------------------------------------------ +
264 rem |<                                FI2MkExp                                >|
266 rem + ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ +
268 rem |                  Create a PSION-compatible export file                   |
270 rem |                                                                          |
272 rem | Prepends an index record to group extensions.                            |
274 rem | Import into Archive, Abacus (max 255 records!), Excel, etc to make your  |
276 rem | adjustments                                                              |
278 rem |                                                                          |
280 rem | dl$ is to set export field delimeter. Comma for PSION, tab works better  |
282 rem | for the likes of Excel.                                                  |
284 rem + ------------------------------------------------------------------------ +
286 rem | V0.01, pjw, 2021 Dec 17                                                  |
288 rem | V0.02, pjw, 2022 Jan 08, added index                                     |
290 rem + ------------------------------------------------------------------------ +
292 :
294 def fn FI2MkExp(fnm$, dl$)
296 loc i%, co, n%, d$, x$(6), xt$(cnt%, 6)
298 x$ = ''
300 for i% = 0 to cnt%
302  if Exts$(i%) <> x$ then
304   if Exts$(i%) = "" then
306    n% = n% + 1
308    xt$(i%) = x$ & chr$(n%)
310   else
312    x$ = Exts$(i%): n% = 65
314    xt$(i%) = x$ & chr$(n%)
316   endif
318  endif
320 endfor i%
322 :
324 d$ = '"' & dl$ & '"'
326 co = fop_over(fnm$): if co < 0: ret co
328 bput#co; 34,'xt$',d$,'Ext$',d$,'Type',d$,'Flags$',d$,'Action$',d$,'Cjob$',d$,'Thing$',d$,'Xfile$',d$,'Command$', 34, 10
330 :
332 for i% = 0 to cnt%
334  print#co; '"'; xt$(i%);'"'; dl$;
336  print#co; '"'; Exts$(i%);'"'; dl$;
338  print#co; Shex(hex(Flgs$(i%, 1 to 4))); dl$; '"$';
340  print#co; Flgs$(i%, 5 to 8);  d$;
342  print#co; Acts$(i%);          d$;
344  print#co; Jobs$(i%);          d$;
346  print#co; Thgs$(i%);          d$;
348  print#co; Fnms$(i%);          d$;
350  print#co; Decode$(Cmds$(i%)); '"'
352 endfor i%
354 close#co
356 ret 0
358 enddef FI2MkExp
360 :
362 :
364 def fn FI2MkData(fnm$)
366 loc i%, co
368 co = fop_over(fnm$): if co < 0: ret co
370 bput#co; 'rem FI2 data for ' & id$, 10, 58, 10
372 bput#co; "rem  Ext, Type, Flags, Action, Cjob, Thing, Xfile, Command", 10
374 :
376 for i% = 0 to cnt%
378  print#co; 'data'! '"'; Exts$(i%);         '", ';
380  print#co; Shex(hex(Flgs$(i%, 1 to 4)));    ', ';
382  print#co; '$'; Flgs$(i%, 5 to 8);          ', ';
384  print#co; '"'; Acts$(i%);                 '", ';
386  print#co; '"'; Jobs$(i%);                 '", ';
388  print#co; '"'; Thgs$(i%);                 '", ';
390  print#co; '"'; Fnms$(i%);                 '", ';
392  print#co; Decode$(Cmds$(i%))
394 endfor i%
396 close#co
398 ret 0
400 enddef FI2MkData
402 :
404 :
406 def proc FI2read
408 loc lp, i%, c%, n%, x$(4)
410 c% = 0: x$ = ext$
412 rep lp
414  x$ = FI2_NEXT$(x$)
416  IF x$ = '': EXIT lp
418  Exts$(c%) = x$
420  n% = fi2_scan%(x$):             rem RI stack error in FI2_SCAN%!
422  for i% = 1 to n%
424   Thgs$(c%) = fi2_scan$(0, x$, i%)
426   Fnms$(c%) = fi2_scan$(1, x$, i%)
428   Cmds$(c%) = fi2_scan$(3, x$, i%)
430   Jobs$(c%) = fi2_scan$(4, x$, i%)
432   Acts$(c%) = fi2_scan$(5, x$, i%)
434   Flgs$(c%) = fi2_scan$(6, x$, i%)
436   c% = c% + 1
438  endfor i%
440 END REPeat lp
442 enddef FI2read
444 :
446 :
448 def fn FI2getmetrics
450 loc lp, i%, j%, c%, n%, x$(4)
452 c% = 0: x$ = ext$
454 REPeat lp
456  x$ = FI2_NEXT$(x$)
458  IF x$ = '': EXIT lp
460  n% = fi2_scan%(x$):             rem RI stack error in FI2_SCAN%!
462  for i% = 1 to n%
464   for j% = 0 to 5
466    l$ = fi2_scan$(j%, x$, i%)
468    if len(l$) > wid%(j%): wid%(j%) = len(l$)
470   endfor j%
472  endfor i%
474  c% = c% + n%
476 END REPeat lp
478 ret c% - 1
480 enddef FI2getmetrics
482 :
484 :
486 def fn Shex(x)
488 if x > 32767: ret x - 65536
490 ret x
492 enddef Shex
494 :
496 DEFine FuNction Decode$(tx$)
498 LOCal i%, c%, l%, s$
500 s$ = "'"
502 FOR i% = 1 TO LEN(tx$)
504  c% = CODE(tx$(i%))
506  SELect ON c%
508   = 32 TO 191: s$ = s$ & tx$(i%)
510   = REMAINDER: s$ = s$ & "' & chr$(" & c% & ") & '"
512  END SELect
514 END FOR i%
516 l% = LEN(s$)
518 IF l% > 2 THEN
520  IF s$(l% - 3 TO l%) = " & '": RETurn s$(1 TO l% - 4)
522 END IF
524 RETurn s$ & "'"
526 END DEFine Decode$
528 :
530 :
